
//////////////////////////////////////////////////////////////////////
// Simple critical section base class.
#ifdef WINDOWS
class CCritical
{
public:
	CCritical()             { InitializeCriticalSection(&m_cs); }
	virtual ~CCritical()    { DeleteCriticalSection    (&m_cs); }

protected:
	void Lock  (void)       { EnterCriticalSection(&m_cs); }
	void Unlock(void)       { LeaveCriticalSection(&m_cs); }

private:
    CRITICAL_SECTION        m_cs;
};
#else
#include <pthread.h>
class CCritical
{
public:
	CCritical()             { 
        pthread_mutexattr_t     mutexattr; 
        pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE_NP);
        pthread_mutex_init(&m_mutex, &mutexattr); 
        pthread_mutexattr_destroy(&mutexattr);
    }
	virtual ~CCritical()    { pthread_mutex_destroy(&m_mutex); }

protected:
	void Lock  (void)       { pthread_mutex_lock(&m_mutex); }
	void Unlock(void)       { pthread_mutex_unlock(&m_mutex); }

private:
    pthread_mutex_t         m_mutex;
};
#endif

