////////////////////////////////////////////////////////////////////////////////
//
// $Header: //depot/Software/IotRouter/dev/U50Module/include/StdHelper.h#1 $
//
// (c) Copyright 2000-2012, Echelon Corporation.  All Rights Reserved.
//
// StdHelper.h - An include file to help hide the difference between
//               Microsoft's implementation of the C++ Standard Template
//               Library and other versions (on other platforms).
//
////////////////////////////////////////////////////////////////////////////////

#ifndef _STDHELPER_H
#define _STDHELPER_H

#ifdef _MSC_VER
    // Microsoft Compilers

    // Use the STLPORT namespace when using Microsoft Visual C++.
    // This bypasses the Standard C++ Library that comes with VC++,
    // which is not portable at all.
    #define STL_NAMESPACE       STLPORT

    //Ignore a bunch of warnings now generated by new STL/code interaction.
    #pragma warning(disable : 4786)        // name truncated warnings
    //#pragma warning(disable : 4244 4018) //possible loss of data in implicit conversion and signed mismatch warnings.

    #undef min
    #undef max
    #ifndef ILON_PLATFORM
        // These templates are useful for preventing conflicts within
        // ATL files...
        template <class _Tp>
        inline const _Tp& min(const _Tp& __a, const _Tp& __b) {
            return __b < __a ? __b : __a;
        }

        template <class _Tp>
        inline const _Tp& max(const _Tp& __a, const _Tp& __b) {
            return  __a < __b ? __b : __a;
        }
    #endif /* ILON_PLATFORM */

#else /* _MSC_VER */

    // All other compilers.  For us, this means the Tornado development
    // environment, which uses the GNU C++ compiler, version 2.72.
    // No custom namespace needed.

    #define STL_NAMESPACE               std
#endif /* _MSC_VER */


#ifdef STL_NAMESPACE
    #define STL_NAMESPACE_PREFIX        STL_NAMESPACE::
    #define STL_IOS_NAMESPACE_PREFIX    STL_NAMESPACE::ios::
#else
    #define STL_NAMESPACE_PREFIX
    #define STL_IOS_NAMESPACE_PREFIX
#endif


// Standard Containers
#define stdDeque                        STL_NAMESPACE_PREFIX deque
#define stdHashMap                      STL_NAMESPACE_PREFIX hash_map
#define stdList                         STL_NAMESPACE_PREFIX list
#define stdMap                          STL_NAMESPACE_PREFIX map
#define stdMultiMap                     STL_NAMESPACE_PREFIX multimap
#define stdPair                         STL_NAMESPACE_PREFIX pair
#define stdQueue                        STL_NAMESPACE_PREFIX queue
#define stdSet                          STL_NAMESPACE_PREFIX set
#define stdMultiSet                     STL_NAMESPACE_PREFIX multiset
#define stdString                       STL_NAMESPACE_PREFIX string
#define stdVector                       STL_NAMESPACE_PREFIX vector


// Standard Automatic Pointers - #include <memory>
#define stdAutoPtr                      STL_NAMESPACE_PREFIX auto_ptr
#define stdAutoPtrRef                   STL_NAMESPACE_PREFIX auto_ptr_ref


// Standard Algorithms - #include <algorithm>
#define stdAdvance                      STL_NAMESPACE_PREFIX advance
#define stdAllocator                    STL_NAMESPACE_PREFIX allocator
#define stdBinaryFunction               STL_NAMESPACE_PREFIX binary_function
#define stdCopy                         STL_NAMESPACE_PREFIX copy
#define stdEqualTo                      STL_NAMESPACE_PREFIX equal_to
#define stdFind                         STL_NAMESPACE_PREFIX find
#define stdFindIf                       STL_NAMESPACE_PREFIX find_if
#define stdForEach                      STL_NAMESPACE_PREFIX for_each
#define stdHash                         STL_NAMESPACE_PREFIX hash
#define stdLess                         STL_NAMESPACE_PREFIX less
#define stdMakePair                     STL_NAMESPACE_PREFIX make_pair
#define stdMax                          STL_NAMESPACE_PREFIX max
#define stdMin                          STL_NAMESPACE_PREFIX min
#define stdRemove                       STL_NAMESPACE_PREFIX remove
#define stdSwap                         STL_NAMESPACE_PREFIX swap
#define stdTransform                    STL_NAMESPACE_PREFIX transform


// Standard Limits #include <limits>
#define stdNumericLimits                STL_NAMESPACE_PREFIX numeric_limits


// Standard Streams
#define stdIOStream                     STL_NAMESPACE_PREFIX iostream
#define stdIStream                      STL_NAMESPACE_PREFIX istream
#define stdOStream                      STL_NAMESPACE_PREFIX ostream

// Standard File Streams - #include <fstream>
#define stdFStream                      STL_NAMESPACE_PREFIX fstream
#define stdIFStream                     STL_NAMESPACE_PREFIX ifstream
#define stdOFStream                     STL_NAMESPACE_PREFIX ofstream

// Standard String Streams - #include <sstream>
#define stdStringStream                 STL_NAMESPACE_PREFIX stringstream
#define stdIStringStream                STL_NAMESPACE_PREFIX istringstream
#define stdOStringStream                STL_NAMESPACE_PREFIX ostringstream

// Standard Char* Streams - #include <strstream>
// (Deprecated - use String Streams instead)
//#define stdStrStream                  STL_NAMESPACE_PREFIX strstream
//#define stdIStrStream                 STL_NAMESPACE_PREFIX istrstream
//#define stdOStrStream                 STL_NAMESPACE_PREFIX ostrstream


// Standard Stream Manipulators
#define stdDec                          STL_NAMESPACE_PREFIX dec
#define stdEndL                         STL_NAMESPACE_PREFIX endl
#define stdEndS                         STL_NAMESPACE_PREFIX ends
#define stdFixed                        STL_NAMESPACE_PREFIX fixed
#define stdFlush                        STL_NAMESPACE_PREFIX flush
#define stdHex                          STL_NAMESPACE_PREFIX hex
#define stdInternal                     STL_NAMESPACE_PREFIX internal
#define stdLeft                         STL_NAMESPACE_PREFIX left
#define stdNoShowBase                   STL_NAMESPACE_PREFIX noshowbase
#define stdNoShowPoint                  STL_NAMESPACE_PREFIX noshowpoint
#define stdNoUppercase                  STL_NAMESPACE_PREFIX nouppercase
#define stdOct                          STL_NAMESPACE_PREFIX oct
#define stdRight                        STL_NAMESPACE_PREFIX right
#define stdScientific                   STL_NAMESPACE_PREFIX scientific
#define stdSetFill                      STL_NAMESPACE_PREFIX setfill
#define stdSetPrecision                 STL_NAMESPACE_PREFIX setprecision
#define stdSetW                         STL_NAMESPACE_PREFIX setw
#define stdShowBase                     STL_NAMESPACE_PREFIX showbase
#define stdShowPoint                    STL_NAMESPACE_PREFIX showpoint
#define stdUppercase                    STL_NAMESPACE_PREFIX uppercase
#define stdWS                           STL_NAMESPACE_PREFIX ws


// Standard IOS Format Flags
#define flgAdjustField                  STL_IOS_NAMESPACE_PREFIX adjustfield
#define flgBaseField                    STL_IOS_NAMESPACE_PREFIX basefield
#define flgBoolAlpha                    STL_IOS_NAMESPACE_PREFIX boolalpha
#define flgDec                          STL_IOS_NAMESPACE_PREFIX dec
#define flgFixed                        STL_IOS_NAMESPACE_PREFIX fixed
#define flgFloatField                   STL_IOS_NAMESPACE_PREFIX floatfield
#define flgHex                          STL_IOS_NAMESPACE_PREFIX hex
#define flgInternal                     STL_IOS_NAMESPACE_PREFIX internal
#define flgLeft                         STL_IOS_NAMESPACE_PREFIX left
#define flgOct                          STL_IOS_NAMESPACE_PREFIX oct
#define flgRight                        STL_IOS_NAMESPACE_PREFIX right
#define flgScientific                   STL_IOS_NAMESPACE_PREFIX scientific
#define flgShowBase                     STL_IOS_NAMESPACE_PREFIX showbase
#define flgShowPoint                    STL_IOS_NAMESPACE_PREFIX showpoint
#define flgShowPos                      STL_IOS_NAMESPACE_PREFIX showpos
#define flgSkipWs                       STL_IOS_NAMESPACE_PREFIX skipws
#define flgUnitBuf                      STL_IOS_NAMESPACE_PREFIX unitbuf
#define flgUppercase                    STL_IOS_NAMESPACE_PREFIX uppercase



//////////////////////////////////////////////////////////////////////////////
// Additional STL Helper Functors & Macros

#ifndef EXCLUDE_STLPORT_HELPERS     /* define this symbol if you have problems with these helpers */

    // NOTE: The functors below require that you include the relevant STL
    //       headers (e.g. <algorithm>, <string>) BEFORE including this file.
    //       For now, include them here to avoid having to change all the
    //       places that currently include this file.
    #include <ctype.h>              /* for toupper */
    #include <algorithm>
    #include <string>


    // function object that performs a case-insensitive string compare operation
    // e.g. for use in map type definition (3rd template parameter)
    // where the key needs to be case-insensitive.
    // (also works with char*-style strings)
    struct CaseInsensitiveCompare
    {
        bool operator()(const stdString& x, const stdString& y) const
        { return _stricmp(x.c_str(), y.c_str()) < 0; }

        bool operator()(const char* x, const char* y) const
        { return _stricmp(x, y) < 0; }
    };

    // case insensitive map (for string-like keys)
    #define stdCiMap(_Key, _Tp)         stdMap<_Key, _Tp, CaseInsensitiveCompare>


    // function object that performs a case-insensitive string match operation
    // e.g. for use in hash_map type definition (4th template parameter)
    // NOTE: if using this with a hash_map, you MUST also use a case-insensitive
    // hash algorithm (see below)
    // (also works with char*-style strings)
    struct CaseInsensitiveEqual
    {
        bool operator()(const stdString& x, const stdString& y) const
        { return _stricmp(x.c_str(), y.c_str()) == 0; }

        bool operator()(const char* x, const char* y) const
        { return _stricmp(x, y) == 0; }
    };

    // function object that performs a case-insensitive string hash algorithm
    // e.g. for use in hash_map type definition (3rd template parameter)
    // (also works with char*-style strings)
    struct CaseInsensitiveHash
    {
        size_t operator()(const stdString& x) const
        {
            stdString ux(x);
            (void)stdTransform(ux.begin(), ux.end(), ux.begin(), toupper);
            return stdHash<stdString>()(ux);
        }
    };

    // case insensitive hash_map (for string-like keys)
    #define stdCiHashMap(_Key, _Tp)     stdHashMap<_Key, _Tp, CaseInsensitiveHash, CaseInsensitiveEqual>


    // macro to declare STLPORT hash function helper for keys of the
    // specified simple type (e.g. handles, enums, etc.)
    #define DECLARE_HASH(T)                                             \
        _STLP_BEGIN_NAMESPACE                                           \
            _STLP_TEMPLATE_NULL /*lint -esym(39, stlport::hash<*>) */   \
            struct hash<T>                                              \
            {                                                           \
                size_t operator()(T __x) const                          \
                { return (size_t)__x; }                                 \
            };                                                          \
        _STLP_END_NAMESPACE

    // declare STLPORT hash helper for pointer types (e.g. HANDLE)
    // (STLport versions from V5.1 onwards already define a hash for void*)
    #if _STLPORT_VERSION < 0x510
        DECLARE_HASH(void*);
    #endif /*_STLPORT_VERSION < 0x510*/

#endif  /* EXCLUDE_STLPORT_HELPERS */


#endif /* _STDHELPER_H */
