///////////////////////////////////////////////////////////////////////////////
//
// $Header: //depot/Software/IotRouter/dev/U50Module/include/U50Defs.h#5 $
//
//  Copyright 2013-2013, Echelon Corp.  All rights reserved.
//
// Definitions private to the DLL.
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include "Ldv32.h"
#include "platform.h"
#include "U50Driver.h"

// For now we just run at this baud rate:
#define SMPBAUD						115200

void LDDebugError(LPCSTR szFormat, ... );
void LDDebugInform(LPCSTR szFormat, ... );
void LDDebugTracePacket(LPCSTR szType, BYTE *mp, WORD length);
PORT_HANDLE U50OpenComPort(LPCSTR szComPort);

#define COUNT_WaitObjects		2
//
///////////////////////////////////////////////////////////////////////////////
#define MAXLONMSGNOEX				240
#define MAXLONMSG					1280
// We limit the size of the queues in both directions:
#define DOWNLINK_BUF_COUNT		4
#define UPLINK_BUF_COUNT		16

typedef struct 
{
	BYTE NiCmd;     // Network Interface Command
	BYTE Length;    // size of ExpAppMessage
	BYTE ExpAppMessage[MAXLONMSG];
} LDV_Message, *pLDV_Message;

#define SIZE_CHALLENGE 8
#define SIZE_MIP_KEY  6

// The message structures:
//#define SIZE_ULB  (MAXLONMSG+4)  // size of ul serial buffer (including expansions)
//#define SIZE_DLB  (MAXLONMSG+4)  // size of dl serial buffer (including expansions)
#define SIZE_ULB  ((MAXLONMSG+4)*2)  // size of ul serial buffer (including expansions)
#define SIZE_DLB  ((MAXLONMSG+4)*2)  // size of dl serial buffer (including expansions)

// The full message element as stored in this driver:
typedef struct LONVXD_Buffer {
	BYTE Length;					// size of NiCmd + ExpAppMessage
	BYTE NiCmd;						// Network Interface Command
	BYTE ExpAppMessage[MAXLONMSG+1];	// represents max size plus checksum.
	BYTE pad[3];
} LONVXD_Buffer, *pLONVXD_Buffer;

// Alternate LONVXD_Buffer, but assembled for extension messages
#pragma pack(1)
typedef struct LONVXD_ExtBuffer {
	BYTE Length;						// this would be EXT_LENGTH
	WORD ExtLength;						// remember any endian swapping
	BYTE NiCmd;							// Network Interface Command
	BYTE ExpAppMessage[MAXLONMSG+1];	// represents max size plus checksum.
} LONVXD_ExtBuffer, *pLONVXD_ExtBuffer;

// A short version used for nicbREPLY's
typedef struct LONVXD_BufferShort {
	BYTE Length;					// size of NiCmd + ExpAppMessage
	BYTE NiCmd;						// Network Interface Command
	BYTE ExpAppMessage[1+SIZE_CHALLENGE+1];	// represents max size plus checksum.
} LONVXD_BufferShort, *pLONVXD_BufferShort;

// A short version for local commands:
typedef struct LONVXD_CmdLocal {
	BYTE Length;
	BYTE NiCmd;					// Network Interface Command
} LONVXD_CmdLocal;

#pragma pack()
typedef enum {
	TSTATE_IDLE = 0,
	TSTATE_RUNNING,
	TSTATE_KILL,
	TSTATE_STOPPED
} ThreadState;

//////////////////////////////////////////////////////////////////////
// Millisecond timers
#define ACK_TIMEOUT  		100		// ref: 115200 baud
#define STARTUP_ACK_TIMEOUT	250
#define READ_TIMEOUT		2500
#define BACKOFF_TIMEOUT		5		// ref: 115200 baud

#define S10ESC       0x7E // Slta10 Escape Character
#define S10MASK_CPC  0x0F // mask for CP Code
#define S10MASK_ACK  0x10 // mask for ACK bit
#define S10MASK_SEQN 0xE0 // mask for sequence number
#define S10INC_SEQN  0x20 // sequence number increment

// CPC Codes: These MUST concur with the SLTA's
typedef enum 
{
	CpNull   = 0,
	CpFail   = 1,
	CpMsg   = 2,
	CpMsgReq  = 3,
	CpMsgAck  = 4,

	CpMsgQueueCount,        // Messages prior to this one can be queued.

	CpMsgReject  = 5,
	CpNiCmdShort = 6,
	CpNiResync  = 7,
	CpNiCmdPassw = 8,
	CpNiCallback = 9,
	CpInvalid  = 14,
	CpEscape  = 15
} DCMP_CPCODES;

typedef struct
{
	BYTE	Escape;
	BYTE	Code;
	BYTE	Param;
	BYTE	Csum;
} CodePacket;

#define SIZE_CP ((unsigned int)sizeof(CodePacket))

// Local NI commands
#define nicbMODE			0x40
#define nicbMODE_SSI		0x42
#define nicbRESET			0x50	
#define nicbINITIATE		0x51
#define nicbCHALLENGE		0x52
#define nicbREPLY			0x53
#define nicbACK				0xC0
#define nicbNACK			0xC1
#define nicbPHASE_SET		0xC0	// IO_0 - IO_3 output low
#define nicbMODE_L5			0xD0
#define nicbMODE_L2			0xD1
#define nicbSSTATUS			0xE0

// Receiver States:
typedef enum 
{
	LLPSIdle = 0,  // receiver idle, waiting for a CP
	LLPSReadyRec1,  // CpMsg rx'd, waiting for message packet
	LLPSReadyRec2,  // Partial message packet rx'd
	LLPSRecComplete  // whole message completely read
} DCMPRXSTATE;

// Transmitter States:
typedef enum 
{
	LLPTIdle = 0,  // nothing really.
	LLPTAckWaitCP,  // waiting for ack on a CP.
	LLPTAckWaitMsg,  // waiting for ack on a message.
	LLPTMsgAckWait,  // waiting for CpMsgAck.
	LLPTMsgGo,   // CpMsgAck rx'd go ahead.
	LLPTRspWaitCP  // waiting for uplink local command.
} DCMPTXSTATE;

typedef enum 
{
	RSTSIdle = 0,   // nothing really.
	RSTSVersion,	// query version
	RSTSWait4Status,// wait for query version response
	RSTSInitiate,   // must initiate 
	RSTSWait4Challenge1, // 
	RSTSDoReply,   // must reply
	RSTSDoChallenge,  // 
	RSTSWait4Challenge2,
} DCMPRSTSTATE;

typedef unsigned short DcmpOptions;
#define DCMP_DEFAULT 		0x0000
#define DCMP_BLIND			0x0001
#define DCMP_TRACE_MSG		0x0002	// Trace at API send/rcv message level
#define DCMP_SIM_ERROR		0x0004
#define DCMP_TRACE_ERROR	0x0008	// Trace checksum errors
#define DCMP_L2				0x0010
#define DCMP_TRACE_PKT		0x0020	// Trace all packets
#define DCMP_UNLOCK			0x0400

//////////////////////////////////////////////////////////////////////
#define IDX_CommReadEvent		0
#define IDX_ReadThreadNotifier	1

LPSECURITY_ATTRIBUTES GetNullSecurityAttributes(void);
#define CREATE_EVENT(x)		OsalCreateEvent(&x)
#define SIG_NOTIFIER(x)		if(x) {OsalSetEvent(x);}
#define CLOSE_NOTIFIER(x)	if(x) {OsalDeleteEvent(&x); x = NULL;}
#define WAIT_NOTIFIER(x, y)	OsalWaitForEvent(x, y);
#define GET_MS_TICKS()		OsalGetTickCount()
#define SLEEP_TASK(x)		Sleep(x)

#ifdef  __cplusplus
#include <queue>
typedef stdQueue<pLONVXD_Buffer> t_UDlist;
#else
#ifdef U50_KERNEL
struct data_list {
	struct list_head list;
	pLONVXD_Buffer data;
};
typedef struct data_list t_UDlist;
#else
typedef char* t_UDlist;
#endif
#endif
