#include <linux/types.h>
/******************************************************************************
  Function:  ipv6_send_announcement
   
  Summary:
    This callback is used to send an announcement message.  

  Parameters:
    lsSenderHandle: A handle to the callback object that implements the send 
                    function.
    ltV0msg:        The announcement message, in LTV0 format.
    msgLen:         The message length.

*****************************************************************************/
void ipv6_send_announcement(void *lsMappingHandle, const uint8_t *ltV0msg, uint8_t msgLen)
{
}

/******************************************************************************
  Function:  ipv6_get_arbitrary_source_address
   
  Summary:
    This callback is used to retrieve arbitrary IP address information 
    for a given source address.  

  Parameters:
    lsMappingHandle:        A handle used for LS mapping 
    pSourceIpAddress:       On input, a pointer the desired (LS derived) source 
                            IP address.  If this IP address cannot be used, 
                            pSourceIpAddress will be updated with the arbitrary
                            IP address to be used instead.
    pDomainId:              The LS domain ID.
    domainIdLen:            The length (in bytes) of the LS domain ID
    pEnclosedSource:        Pointer to a buffer to receive the necessary LS
                            source addressing information (in V1 format) to be 
                            added to the UDP payload, if any
  Return: 
    The length of the additional enclosed source address information

*****************************************************************************/
uint8_t ipv6_get_arbitrary_source_address(void *lsMappingHandle,
                                          uint8_t *pSourceIpAddress, 
                                          const uint8_t *pDomainId, int domainIdLen,
                                          uint8_t *pEnclosedSource)
{
	return 0;
}

/******************************************************************************
  Function:  ipv6_get_arbitrary_dest_address
   
  Summary:
    This callback is used to used by the ls to udp translation layers to retrieve 
    arbitrary IP address information for a given destination address.  

  Parameters:
    lsMappingHandle:        A handle used for LS mapping 
    pDomainId:              The LS domain ID.
    domainIdLen:            The length (in bytes) of the LS domain ID
    subnetId:               The LS destination subnet ID
    nodeId:                 The LS destination node ID
    ipv1AddrFmt:            The LS/IP address format
    pDestIpAddress:         Pointer to a buffer to receive the destination IP
                            address to be used.
    pEnclosedDest:          Pointer to a buffer to receive additional LS
                            destination address information enclosed in the
                            PDU, if any.
  Return: 
    The length of the additional enclosed destination address information
*****************************************************************************/
uint8_t ipv6_get_arbitrary_dest_address(void *lsMappingHandle,
                                        const uint8_t *pDomainId, uint8_t domainLen, 
                                        uint8_t subnetId, uint8_t nodeId, uint8_t ipv1AddrFmt,
                                        uint8_t *pDestIpAddress, uint8_t *pEnclosedDest)
{
	return 0;
}

/******************************************************************************
  Function:  ipv6_set_arbitrary_address_mapping
   
  Summary:
    This callback is used by the ls to udp translation layers to 
    inform the LS/IP mapping layers that a given LS address uses an
    arbitrary IP address.  

  Parameters:
    lsMappingHandle:        A handle used for LS mapping 
    pArbitraryIpAddr:       The arbitrary IP address to use when addressing
                            the LS device.
    pDomainId:              The LS domain ID.
    domainIdLen:            The length (in bytes) of the LS domain ID
    subnetId:               The LS subnet ID
    nodeId:                 The LS node ID

*****************************************************************************/
void ipv6_set_arbitrary_address_mapping(void *lsMappingHandle, const uint8_t *pArbitraryIpAddr, 
                                         const uint8_t *pDomainId, uint8_t domainLen, 
                                         uint8_t subnetId, uint8_t nodeId)
{
}

/******************************************************************************
  Function:  ipv6_set_derived_address_mapping
   
  Summary:
    This callback is used by the ls to udp translation layers to 
    inform the LS/IP mapping layers that a given LS address uses an
    LS derived IP address.  

  Parameters:
    lsMappingHandle:        A handle used for LS mapping 
    pDomainId:              The LS domain ID.
    domainIdLen:            The length (in bytes) of the LS domain ID
    subnetId:               The LS subnet ID
    nodeId:                 The LS node ID

*****************************************************************************/
void ipv6_set_derived_address_mapping(void *lsMappingHandle, 
                                      const uint8_t *pDomainId, uint8_t domainLen, 
                                      uint8_t subnetId, uint8_t nodeId)
{
}

/******************************************************************************
  Function:  ipv6_set_derived_subnets_mapping
   
  Summary:
    This callback is used by the ls to udp translation layers when an 
    SubnetsAddrMapping message is received.

  Parameters:
    lsMappingHandle:        A handle used for LS mapping 
    pDomainId:              The LS domain ID.
    domainIdLen:            The length (in bytes) of the LS domain ID
    set:                    True to set the derived mapping entries, clear to
                            clear the dervived mapping entries.
    pSubneteMap:            Pointer to a bit map of subnets to set or clear.

*****************************************************************************/
void ipv6_set_derived_subnets_mapping(void *lsMappingHandle, 
                                      const uint8_t *pDomainId, uint8_t domainLen, 
                                      uint8_t set, const uint8_t *pSubnets)
{
}

/******************************************************************************
  Function:  ipv6_is_unicast_address_supported
   
  Summary:
    This callback is used by the ls to udp translation layers to 
    determmine whether or not the specified IP address can be used by this
    device as a source address.

  Parameters:
    lsMappingHandle:        A handle used for LS mapping 
    ipAddress:              The LS domain ID.

*****************************************************************************/
uint8_t ipv6_is_unicast_address_supported(void *lsMappingHandle, const uint8_t *ipAddress)
{
	return 0;
}

